@@ -174,28 +174,23 @@ class IdaLoader(fainterp.FaInterp):
         :return: None
         """
         self.verify_project()
-        with open(TEMP_SIG_FILENAME) as f:
-            sig = hjson.load(f)
 
         if ida_kernwin.ask_yn(1, 'Are you sure you want '
                                  'to save this signature?') != 1:
             return
 
-        self.save_signature(sig)
+        self.save_signature(TEMP_SIG_FILENAME)
 
     def find(self, symbol_name, decremental=False):
         """
         Find symbol by name (as specified in SIG file)
         Show an IDA waitbox while doing so
         :param symbol_name: symbol name
-        :param decremental: Should stop before reaching a command with no
-                            results?
         :return: output address list
         """
         ida_kernwin.replace_wait_box('Searching symbol: \'{}\'...'
                                      .format(symbol_name))
-        return super(IdaLoader, self).find(symbol_name,
-                                           decremental=decremental)
+        return super(IdaLoader, self).find(symbol_name)
 
     def get_python_symbols(self, file_name=None):
         """
